local TransState = {
    canvas = love.graphics.newCanvas(),
    rCanvas = love.graphics.newCanvas(),    -- Used to draw the current state during a transition with shader
    isActive = false
}

function TransState:enter(enter_function,cur_state,prev_state)

    --[[
        The `self.isPlaying` check is there for safety.
        Because if the state changes while the transition is not finished
        this will overwrite the canvas into the canvas, which is not possible.
        I will let you find a solution if you mind even if in case
        real it's not supposed to happened. If this is the case the transition will simply not be carried out.
    ]]

    if self.isActive and not self.isPlaying then

        love.graphics.setCanvas(self.canvas)
            love.graphics.clear()
            prev_state.draw(prev_state)
        love.graphics.setCanvas()

        local shader = cur_state.transitionShader

        if shader then
            shader:send("prevTex", self.canvas)
        end

        self.alphaShade = 1
        self.isPlaying = true

    elseif not self.isActive then
        self.isActive = true
    end

    enter_function(cur_state)

end

function TransState:update(update_function,cur_state,gamestate,dt)

    if self.isPlaying then

        self.alphaShade = self.alphaShade - dt

        if self.alphaShade <= 0 then
            self.isPlaying = false
        end

    end

    update_function(cur_state)

end

function TransState:draw(draw_function,cur_state)

    if self.isPlaying then

        local shader = cur_state.transitionShader

        if shader then

            shader:send("time", 1-self.alphaShade)

            love.graphics.setCanvas(self.rCanvas)
            draw_function(cur_state)
            love.graphics.setCanvas()

            love.graphics.setShader(shader)
            love.graphics.draw(self.rCanvas)
            love.graphics.setShader()

        else
            draw_function(cur_state)
            love.graphics.setColor(1,1,1,self.alphaShade)
            love.graphics.draw(self.canvas)
            love.graphics.setColor(1,1,1)
        end

    else
        draw_function(cur_state)
    end

end


local tsFuncs = {
    enter = true,
    draw = true,
    update = true
}

return function(iState)

    local isLoaded = false
    if type(iState) == "string" then
        local file = assert(io.open(iState:gsub("%.","/")..".lua", "rb"))
        iState = assert(loadstring(file:read("*all")))()
        file:close(); isLoaded = true
    end

    local state = {}

    for k, v in pairs(iState) do

        if tsFuncs[k] then
            state[k] = function(...)
                TransState[k](TransState, v, ...)
            end
        else
            state[k] = v
        end

    end

    if isLoaded then iState = nil end

    return state

end


